@@ -6,21 +6,17 @@
 
 #include "loopbackdevicelink.h"
 
+#include "kdeconnectconfig.h"
 #include "loopbacklinkprovider.h"
 
-LoopbackDeviceLink::LoopbackDeviceLink(const QString &deviceId, LoopbackLinkProvider *provider)
-    : DeviceLink(deviceId, provider)
+LoopbackDeviceLink::LoopbackDeviceLink(LoopbackLinkProvider *parent)
+    : DeviceLink(KdeConnectConfig::instance().deviceId(), parent)
 {
 }
 
-QString LoopbackDeviceLink::name()
-{
-    return QStringLiteral("LoopbackLink");
-}
-
 bool LoopbackDeviceLink::sendPacket(NetworkPacket &input)
 {
-    NetworkPacket output((QString()));
+    NetworkPacket output;
     NetworkPacket::unserialize(input.serialize(), &output);
 
     // LoopbackDeviceLink does not need deviceTransferInfo
@@ -34,3 +30,8 @@ bool LoopbackDeviceLink::sendPacket(NetworkPacket &input)
 
     return true;
 }
+
+DeviceInfo LoopbackDeviceLink::deviceInfo() const
+{
+    return KdeConnectConfig::instance().deviceInfo();
+}
